package main

import (
	"flag"
	log "git.randomchars.net/FreeNitori/Log"
	"github.com/BurntSushi/toml"
	"os"
)

var config configPayload
var configPath string
var defaultConfig = configPayload{
	Token:     "TOKEN",
	MessageID: []string{},
	GuildID:   "GUILD_ID",
	Prefix:    "!",
	Store:     "./store",
}

type configPayload struct {
	Token     string
	MessageID []string
	GuildID   string
	Prefix    string
	Store     string
}

func init() {
	flag.StringVar(&configPath, "-t", "ticket.conf", "Specify configuration file location.")
}

func parse() {
	if _, err := toml.DecodeFile(configPath, &config); err != nil {
		if os.IsNotExist(err) {
			var file *os.File
			if file, err = os.Create(configPath); err != nil {
				log.Fatalf("Error while creating configuration file, %s", err)
				os.Exit(1)
			}
			if err = toml.NewEncoder(file).Encode(defaultConfig); err != nil {
				log.Fatalf("Error while encoding default configuration, %s", err)
				os.Exit(1)
			}
			log.Warnf("Default configuration generated at %s, edit before next startup.", configPath)
			os.Exit(1)
		}
		log.Fatalf("Error while decoding configuration file, %s", err)
		os.Exit(1)
	} else {
		log.Infof("Loaded config at %s.", configPath)
	}

	allowedMessages = make(map[string]bool)
	for _, id := range config.MessageID {
		allowedMessages[id] = true
	}
}
