package main

import (
	"flag"
	"fmt"
	log "git.randomchars.net/FreeNitori/Log"
	multiplexer "git.randomchars.net/FreeNitori/Multiplexer"
	"github.com/bwmarrin/discordgo"
	"github.com/sirupsen/logrus"
	"os"
	"os/signal"
	"syscall"
)

var session *discordgo.Session
var m = multiplexer.New()
var system = multiplexer.NewCategory("System", "System-related utilities.")

func main() {
	flag.Parse()
	parse()

	// Ensure store directory
	if _, err := os.ReadDir(config.Store); err != nil {
		if os.IsNotExist(err) {
			log.Warnf("Creating store directory on %s.", config.Store)
			if err = os.MkdirAll(config.Store, 0700); err != nil {
				log.Fatalf("Error creating store directory on %s, %s", config.Store, err)
				os.Exit(1)
			}
		} else {
			log.Fatalf("Store directory inaccessible, %s.", err)
		}
	}

	// Set discordgo log handler
	discordgo.Logger = func(msgL, _ int, format string, a ...interface{}) {
		var level logrus.Level
		switch msgL {
		case discordgo.LogDebug:
			level = logrus.DebugLevel
		case discordgo.LogInformational:
			level = logrus.InfoLevel
		case discordgo.LogWarning:
			level = logrus.WarnLevel
		case discordgo.LogError:
			level = logrus.ErrorLevel
		}
		log.Instance.Log(level, fmt.Sprintf(format, a...))
	}

	// Set command not found handler
	multiplexer.NoCommandMatched = func(context *multiplexer.Context) {
		return
	}

	// Configure session
	if s, err := discordgo.New(); err != nil {
		log.Fatalf("Error while creating session, %s", err)
		os.Exit(1)
	} else {
		session = s
	}
	session.UserAgent = "DiscordBot (ticket-bot)"
	session.Token = "Bot " + config.Token
	session.ShouldReconnectOnError = true
	session.Identify.Intents = discordgo.IntentsAllWithoutPrivileged

	if state, ok := readTicketState(config.Store + "/state"); ok == true {
		ticketInstancesUser = state.TicketInstancesUser
		ticketInstancesChannel = state.TicketInstancesChannel
		log.Infof("Read %v ticket states from %s.", len(ticketInstancesChannel), config.Store + "/state")
	}

	// Open session
	if err := session.Open(); err != nil {
		log.Fatalf("Error while opening session, %s", err)
		os.Exit(1)
	}

	// Setup multiplexer
	m.SessionRegisterHandlers(session)
	m.Prefix = config.Prefix
	m.Categories = append(m.Categories, system)

	// Output message
	log.Infof("Logged in as %s#%s (%s).",
		session.State.User.Username,
		session.State.User.Discriminator,
		session.State.User.ID)
	if application, err := session.Application("@me"); err != nil {
		log.Fatalf("Unable to get application, %s", err)
		os.Exit(1)
	} else {
		log.Infof("Invite URL: https://discord.com/oauth2/authorize?client_id=%s&scope=bot&permissions=8", application.ID)
	}

	// Signal handling
	signalChannel := make(chan os.Signal, 1)
	signal.Notify(signalChannel, syscall.SIGINT, syscall.SIGTERM, syscall.SIGHUP, os.Interrupt, os.Kill)
	var exit int
	func() {
		for {
			currentSignal := <-signalChannel
			switch currentSignal {
			case os.Interrupt:
				exit = 0
				println()
				log.Info("Gracefully exiting.")
				return
			default:
				exit = 0
				log.Info("Gracefully exiting.")
				return
			}
		}
	}()

	switch exit {
	case 0:
		cleanup()
	default:
		os.Exit(exit)
	}
}
